## Guiding test scenario. To be started from SV v0.12 or later.
## 
## Written  by A.Tokovinin, Nov. 11, 2003
########################################################################
# Global control variables:


set CENTERED 0
set TEND 0
set MEASURING 0
set OVER 0
set WTIME 0

# Set telescope coordinates:
# cmd TLSP set longitude="[CIRC longitude]" latitude="[CIRC latitude]"
########################################################################
# Import the configuration file parameters on place, to release the 
# CFG errors immediately
set wait_time [SV wait_time]
set n_try [CENT n_try]
set work_zone [CENT work_zone]
set cent_zone [CENT cent_zone]
set alpha [CENT alpha]
set cent_period [CENT period]
set shift_step [TLSP shift_step]
set shift_max [TLSP shift_max]
set bkgr_shift_ra [TLSP bkgr_shift_ra]
set bkgr_shift_dec [TLSP bkgr_shift_dec]
########################################################################
# after-scripts identifiers for cancelling:
set background_after 0
set guider_after 0

########################################################################
# Procedures:

# Getting the new object 
# Affects: OBJM(RA), OBJM(DEC), OVER and TEND
# we do not care on which star the telescope is actually pointed, for these tests
proc GETOBJECT {} {
        global TEND
	add_log "GETOBJECT"
	cmd OBJM run ra dec tvis hourh object
	# If there is no object - nothing to observe!:
	if {[OBJM status]=="NOSTAR"} {
		set TEND 0
		return
	}
	cmd TLSP set ra="[OBJM ra]" dec="[OBJM dec]"
	set tvis [OBJM tvis]
        add_log "Tvis= $tvis seconds"
 	set TEND [expr [clock seconds]+$tvis]
	cmd DIMM set object=[OBJM object]
}
# ##########################################################
# Returns the next signs of shifts along the spiral:
# @param init 1 for initialization of spiral movement
# @return [list RA-shift-sign DEC-shift-sign]
set sp_step 0
set sp_rad 0
proc get_spiral {{init 0}} {
	global sp_step sp_rad
	if {$init} {
		set sp_step 0
		set sp_rad 0
		return [list 0 0]
	} else {
		if {$sp_step==8*$sp_rad} {
			incr sp_rad
			set sp_step 0
			return [list 1 0]
		} else {
			set st [expr ($sp_step/$sp_rad) % 8]
			set l [\
			switch $st {
				0 - 7 {list 0 1} 1 - 2 {list -1 0} 3 - 4 {list 0 -1} \
				5 - 6 {list 1 0}
			}]
			incr sp_step
			return $l
		}
	}
}

# Searching the star around pointing position
# Affects: CENTERED
proc SEARCH {} {
	global CENTERED sp_rad shift_step shift_max
	add_log "SEARCH"
	set CENTERED 0
	GUIDER
	# Initialize spiral:
	get_spiral 1
	while {([CENT dra]==999 || [CENT ddec]==999) && 
		[expr $sp_rad*$shift_step]<=$shift_max} {
		if {$sp_rad} {
			set shra [expr [lindex $shifts 0]*$shift_step]
			set shdec [expr [lindex $shifts 1]*$shift_step]
			cmd TLSP run dra=$shra ddec=$shdec
		}
		GUIDER
		# Get the current shift along spiral
		set shifts [get_spiral]
	}
}
##############################################################################
# Iterative centering the star
# Uses: MEASURING
# Affects: CENTERED
# @param auto flag of auto-restart with help of after-script. 
# NOTE: call as "GUIDER" if you want it working nonwithstanding MEASURING=0.
# after-script must call it as "GUIDER 1"
proc GUIDER {{auto 0}} {
	global MEASURING n_try cent_zone work_zone alpha cent_period CENTERED
	global guider_after
	if {!$auto} { 
		add_log "GUIDER non-auto start" 
	} else {
		if {!$MEASURING} return
	        add_log "GUIDER self-start" 
	}
	set try 0
        set delay1 2000

	cmd CENT get dra ddec
# Insert 1-s delay:
#    after 1000; update
	while {[CENT dra]!=999 && [CENT ddec]!=999 && ($MEASURING || !$auto) && \
			([expr abs([CENT dra])]>$cent_zone || \
			[expr abs([CENT ddec])]>$cent_zone) && \
			$try<$n_try} {
	        set corra [expr (-1)*[CENT dra] * $alpha]
		set cordec [expr (-1)*[CENT ddec] * $alpha]
	    set corrci [cmd CORR run dra=$corra ddec=$cordec]
            wait_cmd $corrci
	    set centci [cmd CENT get dra ddec]
            wait_cmd $centci
# Insert  delay:
#        set till [expr [clock seconds] + $delay1]
#	    while {[clock seconds]<$till} {after 1000; update}
#    after 1000; update
		incr try
	}
	set CENTERED [expr abs([CENT dra])<=$work_zone && abs([CENT ddec])<=$work_zone]
#	cmd TLSP get ra dec tvis
add_log "CENTERED->$CENTERED"
	if {$MEASURING} {
		set guider_after [after [expr int($cent_period*1000.)] { GUIDER 1 }]
	}
}
##############################################################################
# Compute the time rest for observations, in [sec]
# @param Limiting hour angle in decimal hours
# @return time rest for observations in seconds
proc getwtime {} {
	global TEND
	if {[set t [expr $TEND - [clock seconds]]]>0} {
		return $t
	} else { return 0}
}

# Stop running (or not) detectors
proc STOP {} {
	global background_after guider_after
	add_log "STOP"
	cmd MASS stop now
	cmd DIMM stop now
	after cancel $background_after
	after cancel $guider_after
}

# Wait some time before "situation becomes better"
# @param dt waiting time
proc wait {} {
	global wait_time
	wait_sec $wait_time
}
########################################################################
## Now the scenario body itself goes:
########################################################################

# Infinite loop of targets observations:
while 1 {
	GETOBJECT

	set WTIME [getwtime]

	if {$WTIME<=0} {
		wait
		continue	
	}
	
#	SLEWTEL $OVER
	GUIDER
        set MEASURING 1
	set WTIME [getwtime]

	if {$WTIME<=0 || !$CENTERED}  {
		wait
		continue	
	}
	
	set BG_NEED 1
	while {$WTIME>0 && $CENTERED} {
		if {$BG_NEED} {
#			BACKGROUND
			set MEASURING 1
		#	GUIDER
			# reset command identifiers for DIMM and MASS to prevent GET DATA:
			# set massci [set dimmci -1]
		}
#		MAINMETER
                add_log "Guide started from main loop every 10s"
                GUIDER
                wait_sec 10;
         
		set WTIME [getwtime]
	}
	
	set MEASURING 0
	
	STOP	
}

